--[[ Changelog
V 2.31: Added the missing revealer component
V 2.3 : Added an option for custom FOV (like in my other mod "Wider FOV")
		Added support  for the "Sea Sack"
V 2.22: On/Off option for "Storable Chef Pouch"
		Fixed a glitch caused by the volcanic update where the piggyback would make you faster
V 2.21: Fixed a crash occuring upon dying
V 2.2 :	Added support for the "Life Jacket"
		A skeleton spawns where you died (as intended)
		The construction amulet now discounts recipes when you're using the amulet slot
		Added color indicators for overheating, freezing and critical naughtiness
V 2.1 : Added support for the "Chef Pouch"
V 2.02: Container size bugfix
V 2.0 : No more overrides. All file modified through modmain
		Rewrote inventorybar and modmain to be clearer
V 1.32: On/Off option for custom slot colors
V 1.31: Fixed boat badges and textures
V 1.2 : Bugfixes
V 1.1 : Shipwrecked update
V 1.0 : Release version

Created By Kiopho
--------------------------------------------------------------------------------------------------------------------------]]

Assets =
{
	Asset("ATLAS", "images/container.xml"),
	Asset("ATLAS", "images/equipslots.xml"),
	Asset("ATLAS", "images/inv_slot_spoiled.xml"),
	Asset("ATLAS", "images/inventory_bg15.xml"),
	Asset("ATLAS", "images/inventory_bg20.xml"),
	Asset("ATLAS", "images/inventory_bg20_4.xml"),
	Asset("ATLAS", "images/inventory_bg20_3.xml"),
	Asset("ATLAS", "images/inventory_bg25.xml"),
	Asset("ATLAS", "images/inventory_bg25_4.xml"),
	Asset("ATLAS", "images/inventory_bg25_3.xml"),
	Asset("ATLAS", "images/inventory_bg55.xml"),
	Asset("ATLAS", "images/krampus_sack_bg.xml"),
}

--------------------------------------------------------------------------------------------------------------------------

_G = GLOBAL
Vector3 = _G.Vector3

_G.package.loaded["rpghud.modenv"] = env

local AoSActive = false
for _, moddir in ipairs(_G.KnownModIndex:GetModsToLoad()) do
    if _G.KnownModIndex:GetModInfo(moddir).name == "Always On Status" then
		AoSActive = true
    end
end

local mod_ver_15 = (GetModConfigData("Mod_Version")=="15")
local mod_ver_20 = (GetModConfigData("Mod_Version")=="20")
local mod_ver_25 = (GetModConfigData("Mod_Version")=="25")
local mod_ver_55 = (GetModConfigData("Mod_Version")=="55")

local bp_storable_yes = (GetModConfigData("Storable_Backpack")=="yes")
local storable_no = (GetModConfigData("Storable_Backpack")=="no")

local icepack_storable_yes = (GetModConfigData("Storable_Icepack")=="yes")
local icepack_storable_no = (GetModConfigData("Storable_Icepack")=="no")

local spicepack_storable_yes = (GetModConfigData("Storable_Spicepack")=="yes")
local spicepack_storable_no = (GetModConfigData("Storable_Spicepack")=="no")

local bp_bigger_yes = (GetModConfigData("Bigger_Backpack")=="yes")
local bp_bigger_no = (GetModConfigData("Bigger_Backpack")=="no")

local speed_penalty_yes = (GetModConfigData("Speed_Penalty")=="yes")
local speed_penalty_no = (GetModConfigData("Speed_Penalty")=="no")

local container_bigger_yes = (GetModConfigData("Bigger_Container")=="yes")
local container_bigger_no = (GetModConfigData("Bigger_Container")=="no")

local shortcuts_hide = (GetModConfigData("Show_Shortcuts")=="no")
local shortcuts_show = (GetModConfigData("Show_Shortcuts")=="yes")

local backpack_yes = (GetModConfigData("Backpack_Slot")=="yes")
local backpack_no = (GetModConfigData("Backpack_Slot")=="no")

local amulet_yes = (GetModConfigData("Amulet_Slot")=="yes")
local amulet_no = (GetModConfigData("Amulet_Slot")=="no")

local custom_slot_color = (GetModConfigData("Custom_Slot_Color")=="yes")
local no_custom_slot_color = (GetModConfigData("Custom_Slot_Color")=="no")

local FOVdefault = (GetModConfigData("FOV")=="default")
local FOV36 = (GetModConfigData("FOV")=="FOV36")
local FOV37 = (GetModConfigData("FOV")=="FOV37")
local FOV38 = (GetModConfigData("FOV")=="FOV38")
local FOV39 = (GetModConfigData("FOV")=="FOV39")
local FOV40 = (GetModConfigData("FOV")=="FOV40")
local FOV41 = (GetModConfigData("FOV")=="FOV41")
local FOV42 = (GetModConfigData("FOV")=="FOV42")
local FOV43 = (GetModConfigData("FOV")=="FOV43")
local FOV44 = (GetModConfigData("FOV")=="FOV44")
local FOV45 = (GetModConfigData("FOV")=="FOV45")

--------------------------------------------------------------------------------------------------------------------------

-- Custom Slot Background Color
local function TintingPostInit(self, num, atlas, bgim, owner, container)
	if container.widgetbgimagetint then
		self.bgimage:SetTint(container.widgetbgimagetint.r, container.widgetbgimagetint.g, container.widgetbgimagetint.b, container.widgetbgimagetint.a)
    end
end

AddClassPostConstruct("widgets/invslot", TintingPostInit)
--

-- Spoiled Food Slot Background Fix
local function ItemTilePostInit(self, invitem)
	self.bg:SetTexture("images/inv_slot_spoiled.xml", "inv_slot_spoiled.tex")
end

AddClassPostConstruct("widgets/itemtile", ItemTilePostInit)
--

-- Add Amulet Slot
if amulet_yes then
	table.insert(_G.EQUIPSLOTS, "NECK")
	_G.EQUIPSLOTS.NECK = "neck"
end
--

-- Add Backpack Slot
if backpack_yes then
	table.insert(_G.EQUIPSLOTS, "PACK")
	_G.EQUIPSLOTS.PACK = "pack"
end
--

-- Inventory Size
local function InventorySize(component,inst)

	local packslot = backpack_yes and 1 or 0
	local amuletslot = amulet_yes and 1 or 0
	local num_equip_slots = 3 + packslot + amuletslot
	local max_slots = mod_ver_15 and 15 or mod_ver_20 and 20 or mod_ver_25 and 25 or mod_ver_55 and 55
	
	inst.components.inventory.numequipslots = num_equip_slots
	inst.components.inventory.maxslots = max_slots
	
end

AddComponentPostInit("inventory", InventorySize)
--

-- Inventory Bar
local function InventorybarRework(self)

	local InvSlot = _G.require "widgets/invslot"
	local Image = _G.require "widgets/image"
	local Widget = _G.require "widgets/widget"
	local EquipSlot = _G.require "widgets/equipslot"
	local ItemTile = _G.require "widgets/itemtile"
	
	local HUD_ATLAS = "images/hud.xml"
	local W = 64
	local SEP = 7
	local YSEP = 8
	local INTERSEP = 11
	
	local row_offset = mod_ver_15 and 0 or (W + SEP)
	
	self.out_pos = Vector3(0, W, 0)
	self.in_pos = Vector3(0, W + W + SEP, 0)
	
	self.bg:Kill()
	self.bgcover:Kill()
	
	self.bg = self.root:AddChild(Widget("bg"))
	self.bg:SetPosition(Vector3(0, -SEP/2, 0))
	
	self.bg15 = self.bg:AddChild(Image("images/inventory_bg15.xml", "inventory_bg15.tex"))
	self.bg15:SetScale(1.113,1,1)
	self.bg15:SetPosition(-1,-54,0)
	
	self.bg20 = self.bg:AddChild(Widget("bg20"))
	self.bg20:SetScale(1.377,1,1)
	self.bg20:SetPosition(2.5,-216,0)
	
	self.bg20_5 = self.bg20:AddChild(Image("images/inventory_bg20.xml", "inventory_bg20.tex"))
	self.bg20_5:SetVRegPoint(_G.ANCHOR_BOTTOM)
	
	self.bg20_4 = self.bg20:AddChild(Image("images/inventory_bg20_4.xml", "inventory_bg20_4.tex"))
	self.bg20_4:SetVRegPoint(_G.ANCHOR_BOTTOM)
	
	self.bg20_3 = self.bg20:AddChild(Image("images/inventory_bg20_3.xml", "inventory_bg20_3.tex"))
	self.bg20_3:SetVRegPoint(_G.ANCHOR_BOTTOM)
	
	self.bg25 = self.bg:AddChild(Widget("bg25"))
	self.bg25:SetScale(1.38,1,1)
	self.bg25:SetPosition(-0.5,-114,0)
	
	self.bg25_5 = self.bg25:AddChild(Image("images/inventory_bg25.xml", "inventory_bg25.tex"))
	self.bg25_5:SetVRegPoint(_G.ANCHOR_BOTTOM)
	
	self.bg25_4 = self.bg25:AddChild(Image("images/inventory_bg25_4.xml", "inventory_bg25_4.tex"))
	self.bg25_4:SetVRegPoint(_G.ANCHOR_BOTTOM)
	
	self.bg25_3 = self.bg25:AddChild(Image("images/inventory_bg25_3.xml", "inventory_bg25_3.tex"))
	self.bg25_3:SetVRegPoint(_G.ANCHOR_BOTTOM)
	
	self.bg55 = self.bg:AddChild(Image("images/inventory_bg55.xml", "inventory_bg55.tex"))
	self.bg55:SetVRegPoint(_G.ANCHOR_BOTTOM)
	self.bg55:SetScale(1.654,1,0)
	self.bg55:SetPosition(-2,-116,0)
	
	if _G.EQUIPSLOTS.NECK then
		self:AddEquipSlot(_G.EQUIPSLOTS.NECK, "images/equipslots.xml", "equip_slot_neck.tex")
	end
	
	if _G.EQUIPSLOTS.PACK then
		self:AddEquipSlot(_G.EQUIPSLOTS.PACK, "images/equipslots.xml", "equip_slot_pack.tex")
	end
	
	local function BackpackGet(inst, data)
		local owner = inst.components.inventoryitem and inst.components.inventoryitem.owner
		
		if owner then
			local inv = owner.HUD.controls.inv
			if inv then
				inv:OnItemGet(data.item, inv.backpackinv[data.slot], data.src_pos)
			end
		end		
	end

	local function BackpackLose(inst, data)
		local owner = inst.components.inventoryitem and inst.components.inventoryitem.owner
		if owner then
			local inv = owner.HUD.controls.inv
			if inv then
				inv:OnItemLose(inv.backpackinv[data.slot])
			end
		end		
	end
	
	function self:Rebuild()
	
		if self.cursor then
			self.cursor:Kill()
			self.cursor = nil
		end
		
		if self.toprow then
			self.toprow:Kill()
		end

		if self.bottomrow then
			self.bottomrow:Kill()
		end

		self.toprow = self.root:AddChild(Widget("toprow"))
		self.bottomrow = self.root:AddChild(Widget("toprow"))
		self.bottomrow:SetPosition(0, -W -SEP, 0) --MOD

		self.inv = {}
		self.equip = {}
		self.backpackinv = {}

		local y = 0 --MOD
		local eslot_order = {}

		local num_slots = self.owner.components.inventory:GetNumSlots()
		local num_equip = #self.equipslotinfo
		
		-->MOD
		if mod_ver_15 then
		
			INTERSEP = 15
			
			local num_intersep = 3
			local total_w = (num_slots + num_equip)*(W) + (num_slots + num_equip - 1 - num_intersep)*(SEP) + (INTERSEP*num_intersep)
			
			for k, v in ipairs(self.equipslotinfo) do
				local slot = EquipSlot(v.slot, v.atlas, v.image, self.owner)
				self.equip[v.slot] = self.toprow:AddChild(slot)
				local x = -total_w/2 + W/2 + 3*(INTERSEP - SEP) + (k+15-1)*W + (k+15-1)*SEP
				slot:SetPosition(x,y,0)
				table.insert(eslot_order, slot)
				
				local item = self.owner.components.inventory:GetEquippedItem(v.slot)
				if item then
					slot:SetTile(ItemTile(item, self.owner.components.inventory))
				end

			end

			for k = 1,num_slots do
				local slot = InvSlot(k, HUD_ATLAS, "inv_slot.tex", self.owner, self.owner.components.inventory)
				self.inv[k] = self.toprow:AddChild(slot)
				local interseps = math.floor((k-1) / 5)
				local x = -total_w/2 + W/2 + interseps*(INTERSEP - SEP) + (k-1)*W + (k-1)*SEP
				slot:SetPosition(x,y,0)
				
				local item = self.owner.components.inventory:GetItemInSlot(k)
				if item then
					slot:SetTile(ItemTile(item, self.owner.components.inventory))
				end
				
			end
			
			self.bg15:SetScale(1.113*((20-5+num_equip)/20),1,0)
			self.bg20:Hide()
			self.bg25:Hide()
			self.bg55:Hide()
			
		elseif mod_ver_20 then
		
			INTERSEP = 16

			local num_intersep = 3
			local total_e = num_equip*W + (num_equip - 1)*SEP
			local total_w = (num_slots)*(W) + (num_slots - 1 - num_intersep)*(SEP) + (INTERSEP*num_intersep)
			
			for k, v in ipairs(self.equipslotinfo) do
				local slot = EquipSlot(v.slot, v.atlas, v.image, self.owner)
				self.equip[v.slot] = self.toprow:AddChild(slot)
				local x = -total_e/2 + W/2 + (k-1)*W + (k-1)*SEP
				slot:SetPosition(x,y + row_offset + 2.5,0)
				table.insert(eslot_order, slot)
				
				local item = self.owner.components.inventory:GetEquippedItem(v.slot)
				if item then
					slot:SetTile(ItemTile(item, self.owner.components.inventory))
				end

			end    

			for k = 1,num_slots do
				local slot = InvSlot(k, HUD_ATLAS, "inv_slot.tex", self.owner, self.owner.components.inventory)
				self.inv[k] = self.toprow:AddChild(slot)
				local interseps = math.floor((k-1) / 5)
				local x = -total_w/2 + W/2 + interseps*(INTERSEP - SEP) + (k-1)*W + (k-1)*SEP
				slot:SetPosition(x,y,0)
				
				local item = self.owner.components.inventory:GetItemInSlot(k)
				if item then
					slot:SetTile(ItemTile(item, self.owner.components.inventory))
				end
				
			end
			
			self.bg15:Hide()
			self.bg25:Hide()
			self.bg55:Hide()
			
			if num_equip == 3 then
				self.bg20_5:Hide()
				self.bg20_4:Hide()
			elseif num_equip == 4 then
				self.bg20_5:Hide()
				self.bg20_3:Hide()
			elseif num_equip == 5 then
				self.bg20_4:Hide()
				self.bg20_3:Hide()
			end
			
		elseif mod_ver_25 then
		
			local num_intersep = 4
			local total_e = num_equip*W + (num_equip - 1)*SEP
			local total_w = (num_slots)*(W) + (num_slots - 1 - num_intersep)*(SEP) + (INTERSEP*num_intersep)
			
			for k, v in ipairs(self.equipslotinfo) do
				local slot = EquipSlot(v.slot, v.atlas, v.image, self.owner)
				self.equip[v.slot] = self.toprow:AddChild(slot)
				local x = -total_e/2 + W/2 + (k-1)*W + (k-1)*SEP
				slot:SetPosition(x,y + row_offset ,0)
				table.insert(eslot_order, slot)
				
				local item = self.owner.components.inventory:GetEquippedItem(v.slot)
				if item then
					slot:SetTile(ItemTile(item, self.owner.components.inventory))
				end

			end    

			for k = 1,num_slots do
				local slot = InvSlot(k, HUD_ATLAS, "inv_slot.tex", self.owner, self.owner.components.inventory)
				self.inv[k] = self.toprow:AddChild(slot)
				local interseps = math.floor((k-1) / 5)

				local x = -total_w/2 + W/2 + interseps*(INTERSEP - SEP) + (k-1)*W + (k-1)*SEP
				slot:SetPosition(x,y,0)
				
				local item = self.owner.components.inventory:GetItemInSlot(k)
				if item then
					slot:SetTile(ItemTile(item, self.owner.components.inventory))
				end
				
			end
			
			self.bg15:Hide()
			self.bg20:Hide()
			self.bg55:Hide()
			
			if num_equip == 3 then
				self.bg25_5:Hide()
				self.bg25_4:Hide()
			elseif num_equip == 4 then
				self.bg25_5:Hide()
				self.bg25_3:Hide()
			elseif num_equip == 5 then
				self.bg25_4:Hide()
				self.bg25_3:Hide()
			end
			
		elseif mod_ver_55 then
		
			local num_intersep = 5
			local total_e = num_equip*W + (num_equip - 1)*SEP
			local total_w = (num_slots + 5 - 30)*(W) + (num_slots + 5 - 30 - 1 - num_intersep)*(SEP) + (INTERSEP*num_intersep)
			local offset =  total_w/2 - 2.5*W - 2*SEP
			
			
			for k, v in ipairs(self.equipslotinfo) do
				local slot = EquipSlot(v.slot, v.atlas, v.image, self.owner)
				self.equip[v.slot] = self.toprow:AddChild(slot)
				local x = offset - total_e/2 + W/2 + (k-1)*W + (k-1)*SEP
				slot:SetPosition(x,y + row_offset,0)
				table.insert(eslot_order, slot)
				
				local item = self.owner.components.inventory:GetEquippedItem(v.slot)
				if item then
					slot:SetTile(ItemTile(item, self.owner.components.inventory))
				end

			end    

			for k = 1,30 do
				local slot = InvSlot(k, HUD_ATLAS, "inv_slot.tex", self.owner, self.owner.components.inventory)
				self.inv[k] = self.toprow:AddChild(slot)
				local interseps = math.floor((k-1) / 5)
				local x = -total_w/2 + W/2 + interseps*(INTERSEP - SEP) + (k-1)*W + (k-1)*SEP
				slot:SetPosition(x,y,0)
				
				local item = self.owner.components.inventory:GetItemInSlot(k)
				if item then
					slot:SetTile(ItemTile(item, self.owner.components.inventory))
				end
				
			end
			
			for k = 31,55 do
				local slot = InvSlot(k, HUD_ATLAS, "inv_slot.tex", self.owner, self.owner.components.inventory)
				self.inv[k] = self.toprow:AddChild(slot)
				local interseps = math.floor((k-1-30) / 5)
				local x = -total_w/2 + W/2 + interseps*(INTERSEP - SEP) + (k-1-30)*W + (k-1-30)*SEP
				slot:SetPosition(x,y + row_offset,0)
				
				local item = self.owner.components.inventory:GetItemInSlot(k)
				if item then
					slot:SetTile(ItemTile(item, self.owner.components.inventory))
				end
				
			end
			
			self.bg15:Hide()
			self.bg20:Hide()
			self.bg20_4:Hide()
			self.bg20_3:Hide()
			self.bg25:Hide()
			self.bg25_4:Hide()
			self.bg25_3:Hide()
			
		end
		--<MOD
		
		local old_backpack = self.backpack
		if self.backpack then
			self.inst:RemoveEventCallback("itemget", BackpackGet, self.backpack)
			self.inst:ListenForEvent("itemlose", BackpackLose, self.backpack)
			self.backpack = nil
		end

		local new_backpack = self.owner.components.inventory.overflow
		local do_integrated_backpack = _G.TheInput:ControllerAttached() and new_backpack
		
		if self.boatwidget then
			-->MOD
				local boatwidget_pos = self.boatwidget:GetPosition()
				self.boatwidget:SetPosition(boatwidget_pos.x, 67.5 + row_offset, 0)
			--<MOD
		end
		
		if do_integrated_backpack then
			local num = new_backpack.components.container.numslots

			-->MOD
			local total_wb = nil
			local backpack_controller_size_max = 20
			
			if mod_ver_15 then
				backpack_controller_size_max = 15 + num_equip
				INTERSEP = 15
				total_wb = (num_slots + num_equip)*(W) + (num_slots + num_equip - 1 - 3)*(SEP) + 3*INTERSEP
			elseif mod_ver_20 then
				INTERSEP = 16
				total_wb = num_slots*W + (num_slots - 1 - 3)*(SEP) + 3*INTERSEP
			elseif mod_ver_25 then
				total_wb = num_slots*W + (num_slots - 1 - 4)*(SEP) + 4*INTERSEP
			elseif mod_ver_55 then
				total_wb = (num_slots + 5)/2*W + ( (num_slots + 5)/2 - 1 - 5)*SEP + 5*INTERSEP
			end
			
			if num > backpack_controller_size_max then
				
				for k = backpack_controller_size_max + 1, num do
					local item = new_backpack.components.container:GetItemInSlot(k)
					new_backpack.components.container:DropItem(item)
				end
				
				new_backpack.components.container:SetNumSlots(backpack_controller_size_max, num)
				num = backpack_controller_size_max
				
			end
			--<MOD

			for k = 1, num do
				local slot = InvSlot(k, HUD_ATLAS, "inv_slot.tex", self.owner, new_backpack.components.container)
				self.backpackinv[k] = self.bottomrow:AddChild(slot)

				-->MOD
				local interseps_backpack = math.floor((k-1) / 5)
				local x = -total_wb/2 + W/2 + interseps_backpack*(INTERSEP - SEP) + (k-1)*W + (k-1)*SEP
				slot:SetPosition(x,y,0)
				--<MOD
				
				local item = new_backpack.components.container:GetItemInSlot(k)
				if item then
					slot:SetTile(ItemTile(item))
				end
				
			end
			
			self.backpack = self.owner.components.inventory.overflow
			self.inst:ListenForEvent("itemget", BackpackGet, self.backpack)
			self.inst:ListenForEvent("itemlose", BackpackLose, self.backpack)
			
		end

		if old_backpack	and not self.backpack then
			self:SelectSlot(self.inv[1])
			self.current_list = self.inv
		end

		if do_integrated_backpack then
			self.root:SetPosition(self.in_pos)
		else
			self.root:SetPosition(self.out_pos)
		end
		
		self.actionstring:MoveToFront()
		
		self:SelectSlot(self.inv[1])
		self.current_list = self.inv
		self:UpdateCursor()
		
		if self.cursor then
			self.cursor:MoveToFront()
		end

		self.rebuild_pending = false
	
	end
	
	local oldOnUpdate = self.OnUpdate
	
	function self:OnUpdate(dt)
	
		oldOnUpdate(self, dt)
		
		self.openhint:Hide()	
	end

end

AddClassPostConstruct("widgets/inventorybar", InventorybarRework)
--

-- Krampus sack
local function krampussackpostinit(inst)
	
	if backpack_yes then
		inst.components.equippable.equipslot = _G.EQUIPSLOTS.PACK
	end
	
	if not bp_bigger_no then
	
		local slotpos = {}
		
		for y = 0, 9 do
			table.insert(slotpos, Vector3(-37, -y*75 + 338 ,0))
			table.insert(slotpos, Vector3(-37 +75, -y*75 + 338 ,0))
		end

		inst.components.container:SetNumSlots(#slotpos)
		inst.components.container.widgetslotpos = slotpos
		inst.components.container.widgetanimbank = nil
		inst.components.container.widgetanimbuild = nil
		inst.components.container.widgetbgatlas = "images/krampus_sack_bg.xml"
		inst.components.container.widgetbgimage = "krampus_sack_bg.tex"
		inst.components.container.widgetpos = Vector3(-76,-70,0)
		
	end
	
	if bp_storable_yes then
		inst.components.inventoryitem.cangoincontainer = true
	end
	
end

AddPrefabPostInit("krampus_sack", krampussackpostinit)
--

-- Piggyback
local function piggybackpostinit(inst)
	
	if backpack_yes then
		inst.components.equippable.equipslot = _G.EQUIPSLOTS.PACK
	end
	
	if not bp_bigger_no then
	
		local slotpos = {}

		for y = 0, 6 do
			table.insert(slotpos, Vector3(-162, -y*75 + 240 ,0))
			table.insert(slotpos, Vector3(-162 +75, -y*75 + 240 ,0))
		end

		inst.components.container:SetNumSlots(#slotpos)
		inst.components.container.widgetslotpos = slotpos
		inst.components.container.widgetanimbank = "ui_krampusbag_2x8"
		inst.components.container.widgetanimbuild = "ui_krampusbag_2x8"
		inst.components.container.widgetpos = Vector3(0,-75,0)
	
	end
	
	if bp_storable_yes then
		inst.components.inventoryitem.cangoincontainer = true
	end
	
end

AddPrefabPostInit("piggyback", piggybackpostinit)

if _G.IsDLCEnabled(_G.CAPY_DLC) then
	TUNING.PIGGYBACK_SPEED_MULT = speed_penalty_no and 0 or -0.1
else
	TUNING.PIGGYBACK_SPEED_MULT = speed_penalty_no and 1 or .9
end
--

-- Backpack
local function backpackpostinit(inst)

	if backpack_yes then
		inst.components.equippable.equipslot = _G.EQUIPSLOTS.PACK
	end

	if not bp_bigger_no then
	
		local slotpos = {}

		for y = 0, 4 do
			table.insert(slotpos, Vector3(-162, -y*75 + 112 ,0))
			table.insert(slotpos, Vector3(-162 +75, -y*75 + 112 ,0))
		end
	
		inst.components.container:SetNumSlots(#slotpos)
		inst.components.container.widgetslotpos = slotpos
		inst.components.container.widgetanimbank = "ui_krampusbag_2x5"
		inst.components.container.widgetanimbuild = "ui_krampusbag_2x5"
		inst.components.container.widgetpos = Vector3(0,-50,0)
	
	end
	
	if bp_storable_yes then
		inst.components.inventoryitem.cangoincontainer = true
	end
	
end

AddPrefabPostInit("backpack", backpackpostinit)
--

-- Icepack
local function icepackpostinit(inst)

	if backpack_yes then
		inst.components.equippable.equipslot = _G.EQUIPSLOTS.PACK
	end

	if icepack_storable_yes then
		inst.components.inventoryitem.cangoincontainer = true
	end
	
	if custom_slot_color then
		inst.components.container.widgetbgimagetint = {r=.44,g=.74,b=1,a=1}
	end
	
end

AddPrefabPostInit("icepack", icepackpostinit)
--

-- Chef Pouch
local function SpicePackRework(inst)

	if backpack_yes then
		inst.components.equippable.equipslot = _G.EQUIPSLOTS.PACK
	end
	
	inst.components.container.widgetpos = Vector3(25,-60,0)
	
	if custom_slot_color then
		inst.components.container.widgetbgimagetint = {r=.44,g=.74,b=1,a=1}
	end
	
	if spicepack_storable_yes then
		inst.components.inventoryitem.cangoincontainer = true
	end
	
end

AddPrefabPostInit("spicepack", SpicePackRework)
--


-- Chester
local function chesterpostinit(inst)
	inst.components.container.widgetbgimagetint = {r=.75,g=.58,b=.44,a=1}
end

if custom_slot_color then
	AddPrefabPostInit("chester", chesterpostinit)
end
--

-- Chests
local function treasurechestpostinit(inst)

	if custom_slot_color then
		inst.components.container.widgetbgimagetint = {r=.82,g=.77,b=.7,a=1}
	end
	
	if not container_bigger_no then
	
		local slotpos = {}

		for y = 3, 0, -1 do
			for x = 0, 3 do
				table.insert(slotpos, Vector3(80*x-80*2+40, 80*y-80*2+40,0))
			end
		end

		inst.components.container:SetNumSlots(#slotpos)
		inst.components.container.widgetanimbank = nil
		inst.components.container.widgetanimbuild = nil
		inst.components.container.widgetbgatlas = "images/container.xml"
		inst.components.container.widgetbgimage = "container.tex"
		inst.components.container.widgetslotpos = slotpos
		
	end

end

AddPrefabPostInit("treasurechest", treasurechestpostinit)
--

-- Ice Box
local function iceboxpostinit(inst)

	if custom_slot_color then
		inst.components.container.widgetbgimagetint = {r=.44,g=.74,b=1,a=1}
	end

	if not container_bigger_no then
	
		local slotpos = {}

		for y = 3, 0, -1 do
			for x = 0, 3 do
				table.insert(slotpos, Vector3(80*x-80*2+40, 80*y-80*2+40,0))
			end
		end

		inst.components.container:SetNumSlots(#slotpos)
		inst.components.container.widgetanimbank = nil
		inst.components.container.widgetanimbuild = nil
		inst.components.container.widgetbgatlas = "images/container.xml"
		inst.components.container.widgetbgimage = "container.tex"
		inst.components.container.widgetslotpos = slotpos

	end
	
end

AddPrefabPostInit("icebox", iceboxpostinit)
--

-- Crock Pot
local function crockpotpostinit(inst)
	inst.components.container.widgetbgimagetint = {r=1,g=.42,b=.33,a=1}
end

if custom_slot_color then
	AddPrefabPostInit("cookpot", crockpotpostinit)
end
--

-- Amulets
local function amuletpostinit(inst)
	inst.components.equippable.equipslot = _G.EQUIPSLOTS.NECK
end

if amulet_yes then
	AddPrefabPostInit("amulet", amuletpostinit)
	AddPrefabPostInit("blueamulet", amuletpostinit)
	AddPrefabPostInit("purpleamulet", amuletpostinit)
	AddPrefabPostInit("orangeamulet", amuletpostinit)
	AddPrefabPostInit("greenamulet", amuletpostinit)
	AddPrefabPostInit("yellowamulet", amuletpostinit)
end
--

-- Craft Tab Fix
local function CTabs(self)

	local oldSetOrientation = self.SetOrientation
	
	function self:SetOrientation(horizontal)
	
		oldSetOrientation(self, horizontal)
	
		local slot_w, slot_h = self.bg:GetSlotSize()
		local but_w, but_h = self.downbutton:GetSize()
	
		if horizontal then
			self.downbutton:SetRotation(90)
			self.downbutton:SetPosition(-self.bg.length/2 - but_w/2 + slot_w/2 +40,0,0)
			self.upbutton:SetRotation(-90)
			self.upbutton:SetPosition(self.bg.length/2 + but_w/2 - slot_w/2 - 40,0,0)
		else
			self.upbutton:SetPosition(0, - self.bg.length/2 - but_h/2 + slot_h/2 + 40,0)
			self.downbutton:SetScale(Vector3(1, -1, 1))
			self.downbutton:SetPosition(0, self.bg.length/2 + but_h/2 - slot_h/2 - 40,0)
		end
	end

end

AddClassPostConstruct("widgets/crafting", CTabs)
--

-- Corner Shortcuts
local function Buttons(self)
	self.pauseBtn:Kill()
	self.minimapBtn:Kill()
	self.rotleft:Kill()
	self.rotright:Kill()
end

if shortcuts_hide then
	AddClassPostConstruct("widgets/mapcontrols", Buttons)
end
--

-- Amulet Resurrection Fix
local function ResurrectableFix(inst)

	if _G.IsDLCEnabled(_G.CAPY_DLC) then
	
		local function TrySpawnSkeleton(inst)
			if inst and inst.last_death_position then

				inst:DoTaskInTime(3, function(inst)
					if not inst:GetIsOnWater(inst.last_death_position.x, inst.last_death_position.y, inst.last_death_position.z) then 
						local skel = _G.SpawnPrefab("skeleton_player")
						if skel then
							skel.Transform:SetPosition(inst.last_death_position.x, inst.last_death_position.y, inst.last_death_position.z)
						end
					end 
					inst.last_death_position = nil
				end)
			end
		end

		inst.FindClosestResurrector = function(self, cause)
			local res = nil
			if self.inst.components.inventory and cause ~= "drowning" then
				local item = nil
				
				if not amulet_yes then
					item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.BODY)
				elseif amulet_yes then
					item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.NECK)
				end
				
				if item and item.prefab == "amulet" then
					return item
				end
			end

			local closest_dist = 0
			local res = nil
			if self.inst.components.inventory then
				local item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.BODY)
				local item_N = nil
				if amulet_yes then
					item_N = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.NECK)
				end
				
				if (self.cantdrown or (item and (item.prefab == "armor_lifejacket" or item.prefab == "amulet")) or (item_N and item_N.prefab == "amulet")) and cause == "drowning" then
					--Find and return the closest beach resurrector 
					for k,v in pairs(_G.Ents) do
						--print(v.prefab)
						if v.prefab == "beachresurrector" then
							local dist = v:GetDistanceSqToInst(self.inst)
							if not res or dist < closest_dist then
								res = v
								closest_dist = dist
							end
						end
					end

					if res then return res end 
				end
			end

			
			for k,v in pairs(_G.Ents) do
				if v.components.resurrector and v.components.resurrector:CanBeUsed() then
					local dist = v:GetDistanceSqToInst(self.inst)
					if not res or dist < closest_dist then
						res = v
						closest_dist = dist
					end
				end
			end

			return res
		end

		inst.CanResurrect = function(self, cause)
			if self.cantdrown and cause == "drowning" then 
				return true 
			end 

			if self.inst.components.inventory then
				local item = nil
				
				if not amulet_yes then
					item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.BODY)
				elseif amulet_yes then
					item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.NECK)
				end
				
				if item and item.prefab == "amulet" then
					return true
				end
			end

			if self.inst.components.inventory then
				local item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.BODY)
				if (self.cantdrown or (item and item.prefab == "armor_lifejacket")) and cause == "drowning" then
					return true
				end
			end

			local res = false

			if _G.SaveGameIndex:CanUseExternalResurector() then
				res = _G.SaveGameIndex:GetResurrector() 
			end

			if res == nil or res == false then
				res = self:FindClosestResurrector()
			end

			if res then
				return true
			end

			return false
		end

		inst.DoResurrect = function(self, cause)
			self.inst:PushEvent("resurrect")
			if self.inst.components.inventory and cause ~= "drowning" then
				local item = nil
				
				if not amulet_yes then
					item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.BODY)
				elseif amulet_yes then
					item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.NECK)
				end
				
				if item and item.prefab == "amulet" then
					self.inst.sg:GoToState("amulet_rebirth")
					if self.inst.components.poisonable and self.inst.components.poisonable:IsPoisoned() then 
						self.inst.components.poisonable:Cure()
					end 
					TrySpawnSkeleton(self.inst)
					return true
				end
			end
			if self.inst.components.inventory then
				local item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.BODY)
				local item_N = nil
				if amulet_yes then
					item_N = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.NECK)
				end
				
				if (self.cantdrown or (item and (item.prefab == "armor_lifejacket" or item.prefab == "amulet")) or (item_N and item_N.prefab == "amulet")) and cause == "drowning" then
					local res = self:FindClosestResurrector(cause)
					if res then 
						res:doresurrect(self.inst)
						if self.inst.prefab == "woodie" then  --Woodie gets this flag set if he drowns due to transforming into the werebeaver, we need to change it back
							self.inst.AnimState:SetBank("wilson")
							self.inst.AnimState:SetBuild(self.inst.prefab)
							self.cantdrown = false 
						end 
						return true
					end
				end
			end
			
			local res = self:FindClosestResurrector()
			if res and res.components.resurrector then
				res.components.resurrector:Resurrect(self.inst)
				TrySpawnSkeleton(self.inst)
				return true
			end

			return false
		end
		
	elseif not _G.IsDLCEnabled(_G.CAPY_DLC) then
	
		inst.FindClosestResurrector = function(self)
			local res = nil
			if self.inst.components.inventory then
				local item = nil
				
				if not amulet_yes then
					item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.BODY)
				elseif amulet_yes then
					item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.NECK)
				end
				
				if item and item.prefab == "amulet" then
					return item
				end
			end

			local Ents = _G.Ents
			local closest_dist = 0
			for k,v in pairs(Ents) do
				if v.components.resurrector and v.components.resurrector:CanBeUsed() then
					local dist = v:GetDistanceSqToInst(self.inst)
					if not res or dist < closest_dist then
						res = v
						closest_dist = dist
					end
				end
			end

			return res
		end

		inst.CanResurrect = function(self)
			if self.inst.components.inventory then
				local item = nil
				
				if not amulet_yes then
					item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.BODY)
				elseif amulet_yes then
					item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.NECK)
				end
				
				if item and item.prefab == "amulet" then
					return true
				end
			end

			local res = false

			if _G.SaveGameIndex:CanUseExternalResurector() then
				res = _G.SaveGameIndex:GetResurrector() 
			end

			if res == nil or res == false then
				res = self:FindClosestResurrector()
			end

			if res then
				return true
			end

			return false
		end

		inst.DoResurrect = function(self)
			self.inst:PushEvent("resurrect")
			if self.inst.components.inventory then
				local item = nil
				
				if not amulet_yes then
					item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.BODY)
				elseif amulet_yes then
					item = self.inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.NECK)
				end
				
				if item and item.prefab == "amulet" then
					self.inst.sg:GoToState("amulet_rebirth")
					return true
				end
			end
			
			local res = self:FindClosestResurrector()
			if res and res.components.resurrector then
				res.components.resurrector:Resurrect(self.inst)
				return true
			end

			return false
		end
		
	end

end

AddComponentPostInit("resurrectable", ResurrectableFix)
--

-- Amulet Resurrection Fix (cont.)
local function NewOnExit(inst)

	inst.components.hunger:SetPercent(2/3)
	inst.components.health:Respawn(TUNING.RESURRECT_HEALTH)
	
	if inst.components.sanity then
		inst.components.sanity:SetPercent(.5)
	end
	
	local item = inst.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.NECK)
	if item and item.prefab == "amulet" then
		item = inst.components.inventory:RemoveItem(item)
		if item then
			item:Remove()
			item.persists = false
		end
	end
	--_G.SaveGameIndex:SaveCurrent()
	inst.HUD:Show()
	_G.TheCamera:SetDefault()
	inst.components.playercontroller:Enable(true)
	inst.AnimState:ClearOverrideSymbol("FX")

end

local function SGWilsonPostInit(sg)
	sg.states["amulet_rebirth"].onexit = NewOnExit
end

if amulet_yes then
	AddStategraphPostInit("wilson", SGWilsonPostInit)
end
--

-- Construction Amulet Fix
local function GreenAmuletFix(self)
	local oldRefresh = self.Refresh
	
	function self:Refresh()
		oldRefresh(self)
		
		equippedBody = _G.GetPlayer().components.inventory:GetEquippedItem(_G.EQUIPSLOTS.NECK)
	end
end

if amulet_yes then
	AddClassPostConstruct("widgets/recipepopup", GreenAmuletFix)
end
--

-- Auto-open Backpack
 local function OpenBackpack(self) 
	
	local oldSetMainCharacter = self.SetMainCharacter

	function self:SetMainCharacter(maincharacter)
		
		oldSetMainCharacter(self, maincharacter)
        
		local bp = maincharacter.components.inventory:GetEquippedItem(_G.EQUIPSLOTS.PACK)
		
		if bp and bp.components.container then
			bp.components.container:Close()
			bp.components.container:Open(maincharacter)
		end
		
	end
	
end

if backpack_yes then
	AddClassPostConstruct("screens/playerhud", OpenBackpack)
end
--

-- Camera Zoom
local function Zoom(self)
	
	function self:UpdateClouds(dt)
	
		if not _G.GetWorld():IsCave() then

			local TheCamera = _G.TheCamera
			local TheMixer = _G.TheMixer
			local easing = _G.require("easing")

			if TheCamera and TheCamera.distance and not TheCamera.dollyzoom then
		
				local dist_percent = (TheCamera.distance - TheCamera.mindist) / (TheCamera.maxdist - TheCamera.mindist)
				local cutoff = .6
				if dist_percent > cutoff then
					if not self.clouds_on then
						TheCamera.should_push_down = false
						self.clouds_on = true
						self.clouds:Show()
						--self.owner.SoundEmitter:PlaySound("dontstarve/common/clouds", "windsound")
						--TheMixer:PushMix("high")
					end
	            
					local p = easing.outCubic( dist_percent-cutoff , 0, 1, 1 - cutoff) 
					--self.clouds:GetAnimState():SetMultColour(1,1,1, p )
					--self.owner.SoundEmitter:SetVolume("windsound",p)
				else
					if self.clouds_on then
						TheCamera.should_push_down = false
						self.clouds_on = false
						self.clouds:Hide()
						--self.owner.SoundEmitter:KillSound("windsound")
						TheMixer:PopMix("high")
					end
				end
				
			end
			
		end
		
	end
	
end

AddClassPostConstruct ("screens/playerhud", Zoom)
--

-- Controller/Crafting Status Numbers Display Fix for AoS
local function StatusDisplayNumFix(self)
	
	local oldCloseControllerInventory = self.CloseControllerInventory
	
	function self:CloseControllerInventory()
		oldCloseControllerInventory(self)
		self.controls:ShowStatusNumbers()
	end
	
end

if AoSActive then
	AddClassPostConstruct("screens/playerhud", StatusDisplayNumFix)
end
--

-- Shipwrecked DLC
if _G.IsDLCEnabled(_G.CAPY_DLC) then

	-- Thatchpack
	local function ThatchpackPostInit(inst)
		
		if backpack_yes then
			inst.components.equippable.equipslot = _G.EQUIPSLOTS.PACK
		end
		
		if bp_storable_yes then
			inst.components.inventoryitem.cangoincontainer = true
		end
	end

	AddPrefabPostInit("thatchpack", ThatchpackPostInit)
	--

	-- Piratepack
	local function PiratepackPostInit(inst)
		
		if backpack_yes then
			inst.components.equippable.equipslot = _G.EQUIPSLOTS.PACK
		end
		
		if bp_storable_yes then
			inst.components.inventoryitem.cangoincontainer = true
		end
	end

	AddPrefabPostInit("piratepack", PiratepackPostInit)
	--

	-- Mailpack
	local function PiratepackPostInit(inst)
		
		if backpack_yes then
			inst.components.equippable.equipslot = _G.EQUIPSLOTS.PACK
		end
		
		if bp_storable_yes then
			inst.components.inventoryitem.cangoincontainer = true
		end
	end

	AddPrefabPostInit("mailpack", PiratepackPostInit)
	--
	
	-- Sea Sack
	local function SeaSackPostInit(inst)
		
		if backpack_yes then
			inst.components.equippable.equipslot = _G.EQUIPSLOTS.PACK
		end
		
		if bp_storable_yes then
			inst.components.inventoryitem.cangoincontainer = true
		end
	end

	AddPrefabPostInit("seasack", SeaSackPostInit)
	--
	
	-- Boat Widget Position Fix
	local function BoatBadgeFix(self)
	
		if _G.GetPlayer().HUD then
	
			local oldOpen = self.Open
		
			function self:Open(container, doer, boatwidget)
		
				oldOpen(self, container, doer, boatwidget)
				
				local boatwidget_pos = self:GetPosition()
				
				if self.container == _G.GetPlayer().components.driver.vehicle then
					if container.prefab == "cargoboat" then
						self:SetPosition(boatwidget_pos.x + 322, boatwidget_pos.y, 0)
					elseif container.prefab == "armouredboat" or container.prefab == "rowboat" then
						self:SetPosition(boatwidget_pos.x + 75, boatwidget_pos.y, 0)
					elseif container.prefab == "lograft" or container.prefab == "raft" then
						self:SetPosition(boatwidget_pos.x, boatwidget_pos.y, 0) -- placeholder, in case these are changed by Klei
					end
				end
			end
		end
	end
	
	AddClassPostConstruct("widgets/containerwidget", BoatBadgeFix)

end
--

-- Custom FOV
if not FOVdefault then

	local FollowCamera = _G.require("cameras/followcamera")
	local Default = FollowCamera.SetDefault

	FollowCamera.SetDefault = function(self)
		Default(self)
		
		if FOV36 then
			self.fov = 36
		elseif FOV37 then
			self.fov = 37
		elseif FOV38 then
			self.fov = 38
		elseif FOV39 then
			self.fov = 39
		elseif FOV40 then
			self.fov = 40
		elseif FOV41 then
			self.fov = 41
		elseif FOV42 then
			self.fov = 42
		elseif FOV43 then
			self.fov = 43
		elseif FOV44 then
			self.fov = 44
		elseif FOV45 then
			self.fov = 45
		end
	end

	local function RevealerFix(inst)
		inst:AddComponent("revealer")
		
		if FOV36 then
			inst.components.revealer.radius = 61.2
		elseif FOV37 then
			inst.components.revealer.radius = 62.4
		elseif FOV38 then
			inst.components.revealer.radius = 63.6
		elseif FOV39 then
			inst.components.revealer.radius = 64.8
		elseif FOV40 then
			inst.components.revealer.radius = 66
		elseif FOV41 then
			inst.components.revealer.radius = 67.2
		elseif FOV42 then
			inst.components.revealer.radius = 68.4
		elseif FOV43 then
			inst.components.revealer.radius = 69.6
		elseif FOV44 then
			inst.components.revealer.radius = 70.8
		elseif FOV45 then
			inst.components.revealer.radius = 72
		end
		
		inst.components.revealer:StartUpdating()
	end

	for k,prefabname in ipairs(CHARACTERLIST) do
		AddPrefabPostInit(prefabname, RevealerFix)
	end

	if _G.MODCHARACTERLIST then
		for k,prefabname in ipairs(_G.MODCHARACTERLIST) do
			AddPrefabPostInit(prefabname, RevealerFix)
		end
	end

end